@@ -66,7 +66,7 @@ icd_children.character <- function(x, ...) {
 #' @export
 icd_children.icd9 <- function(x, short_code = icd_guess_short(x),
                               defined = TRUE, billable = FALSE, ...) {
-  assert(checkmate::checkFactor(x), checkmate::checkCharacter(x))
+  assert(check_factor(x), check_character(x))
   assert_flag(short_code)
   assert_flag(defined)
   assert_flag(billable)
@@ -94,7 +94,7 @@ icd_children.icd10 <- function(x, short_code = icd_guess_short(x), defined, bill
 #' @export
 #' @keywords internal
 icd_children.icd10cm <- function(x, short_code = icd_guess_short(x), defined, billable = FALSE, ...) {
-  assert(checkmate::checkFactor(x), checkmate::checkCharacter(unclass(x)))
+  assert(check_factor(x), check_character(unclass(x)))
   assert_flag(short_code)
   assert_flag(billable)
 
@@ -117,7 +117,8 @@ utils::globalVariables("icd10cm2016")
 icd_children_defined <- function(x)
   UseMethod("icd_children_defined")
 
-#' @describeIn icd_children_defined Internal function to get the children of ICD-10 code(s)
+#' @describeIn icd_children_defined Internal function to get the children of
+#'   ICD-10 code(s)
 #' @param warn single logical value, if \code{TRUE} will generate warnings when
 #'   some input codes are not known ICD-10-CM codes
 #' @param use_cpp single logical flag, whether to use C++ version
@@ -127,13 +128,12 @@ icd_children_defined <- function(x)
 #' microbenchmark::microbenchmark(
 #'   icd:::icd_children_defined.icd10cm("A01"),
 #'   icd:::icd_children_defined_r.icd10cm("A01")
-#' )
-#' }
+# ' )
 #' stopifnot(identical(icd:::icd_children_defined.icd10cm("A00"),
 #'   icd:::icd_children_defined_r.icd10cm("A00")))
+#' }
 #' @keywords internal
 icd_children_defined.icd10cm <- function(x, short_code = icd_guess_short(x), warn = FALSE) {
-
   assert_character(x)
   assert_flag(short_code)
   assert_flag(warn)
@@ -146,44 +146,3 @@ icd_children_defined.icd10cm <- function(x, short_code = icd_guess_short(x), war
   kids <- icd10cm_children_defined_cpp(x)
   as.icd10cm(kids, short_code)
 }
-
-icd_children_defined_r.icd10cm <- function(x, short_code = icd_guess_short(x), warn = FALSE) {
-
-  assert_character(x)
-  assert_flag(short_code)
-
-  x <- trim(x)
-  if (!short_code)
-    x <- icd_decimal_to_short.icd10cm(x)
-
-  # we match twice here, once with %in% and once with match...
-  matches_bool <- x %in% icd::icd10cm2016[["code"]]
-  # if the codes are not in the source file, we ignore, warn, drop silently?
-  if (warn && !all(matches_bool))
-    warning("some values did not match any ICD-10-CM codes: ",
-            paste(x[!matches_bool], collapse = ", "))
-
-  x <- x[matches_bool]
-  matches <- match(x, icd::icd10cm2016[["code"]])
-  last_row <- nrow(icd::icd10cm2016)
-
-  kids <- character(0)
-
-  if (length(x) == 0) {
-    if (length(x) > 0)
-      warning("none of the provided ICD-10 codes matched the canonical list")
-    return(icd10cm(character(0)))
-  }
-
-  for (i in seq_along(x)) {
-    # now the children, assuming the source file is sorted logically, will be
-    # subsequent codes, until a code of the same length is found (and stop one before that)
-    check_row <- matches[i] + 1 # start with code after the match from the input
-    parent_len <- .nc[matches[i]]
-    while (.nc[check_row] > parent_len && check_row != last_row + 1)
-      check_row <- check_row + 1
-
-    kids <- c(kids, icd::icd10cm2016[matches[i]:(check_row - 1), "code"])
-  }
-  as.icd10cm(kids, short_code)
-}
