@@ -32,4 +32,8 @@ on.exit(options(old_warn))
 
 icd:::setup_test_check()
 icd:::show_test_options()
-icd:::my_test_check("deprecated", "Running deprecated tests")
+if (icd:::doing_slow_tests()) {
+  icd:::my_test_check("deprecated", "Running deprecated tests")
+} else {
+  message("skipping deprecated tests because icd.do_slow_tests is FALSE")
+}
