@@ -98,46 +98,6 @@ test_that("deprecated - Elixhauser icd9 mappings are all
               expect_equivalent(icd9GetInvalidMappingShort(elixComorbid), list())
             })
 
-test_that("deprecated - can condense the big lists of comorbidities without errors", {
-  # this is a useful but slow (8s on my PC) test because the data weren't
-  # generated by just expanding base ranges (which is how the condense works in
-  # reverse)
-
-  skip_on_cran()
-  skip_slow_tests()
-
-  for (onlyReal in c(TRUE, FALSE)) {
-    if (onlyReal) {
-      # this is a bit silly now, because deprecated functions always give warnings:
-      expect_that(ahrq <- lapply(ahrqComorbid, icd9CondenseShort, onlyReal = onlyReal),
-                  gives_warning())
-      expect_that(quanDeyo <- lapply(quanDeyoComorbid, icd9Condense, onlyReal),
-                  gives_warning())
-      expect_that(quanElix <- lapply(quanElixComorbid,
-                                     icd9Condense, onlyReal),
-                  gives_warning())
-      expect_that(elix <- lapply(elixComorbid, icd9Condense, onlyReal),
-                  gives_warning())
-    }
-    else {
-      ahrq <- lapply(ahrqComorbid, icd9CondenseShort, onlyReal)
-      quanDeyo <- lapply(quanDeyoComorbid, icd9CondenseShort, onlyReal)
-      quanElix <- lapply(quanElixComorbid, icd9CondenseShort, onlyReal)
-      elix <- lapply(elixComorbid, icd9CondenseShort, onlyReal)
-    }
-
-    expect_is(ahrq, class = "list")
-    expect_is(elix, class = "list")
-    expect_is(quanDeyo, class = "list")
-    expect_is(quanElix, class = "list")
-    # the comorbidity mappings save in \code{data} should not be condensed.
-    expect_false(identical(ahrq, ahrqComorbid))
-    expect_false(identical(elix, elixComorbid))
-    expect_false(identical(quanElix, quanElixComorbid))
-    expect_false(identical(quanDeyo, quanDeyoComorbid))
-  }
-})
-
 test_that("deprecated - Charlson Deyo doesn't double count disease with two severities", {
   expect_false(any(quanDeyoComorbid[["LiverMild"]] %in% quanDeyoComorbid[["LiverSevere"]] ))
   expect_false(any(quanDeyoComorbid[["Cancer"]] %in% quanDeyoComorbid[["Mets"]] ))
@@ -652,13 +612,14 @@ test_that("deprecated - comorbidities created from source data frame coded as fa
 })
 
 test_that("deprecated data aliases are created", {
-  expect_warning(dep_dat <- generate_deprecated_data(save_data = FALSE), regexp = NA)
-
-  # actually save the deprecated data somewhere, and reload it
   tmp_file_path <- tempfile()
   on.exit(unlink(tmp_file_path))
-  generate_deprecated_data(save_data = TRUE, path = tmp_file_path)
-  get(load(tmp_file_path))
+  expect_warning(
+    dep_dat <- generate_deprecated_data(save_data = TRUE,
+                                        path = tmp_file_path,
+                                        compress = FALSE),
+    regexp = NA)
+  load(tmp_file_path)
 
   expect_true(
     setequal(names(dep_dat),
